=== modified file 'NEWS'
--- NEWS	2006-03-15 13:59:19 +0000
+++ NEWS	2006-03-22 12:05:37 +0000
@@ -54,7 +54,7 @@
 
     * New 'reconcile' command will check branch consistency and repair indexes
       that can become out of sync in pre 0.8 formats. (Robert Collins,
-      Aaron Bentley)
+      Daniel Silverstone)
 
     * New 'bzr init --format' and 'bzr upgrade --format' option to control 
       what storage format is created or produced.  (Robert Collins, 
@@ -62,6 +62,18 @@
 
     * Add parent location to 'bzr info', if there is one.  (Olaf Conradi)
 
+    * New developer commands 'weave-list' and 'weave-join'.  (Martin Pool)
+
+    * New 'init-repository' command, plus support for repositories in 'init'
+      and 'branch'
+
+  CHANGES:
+
+    * Commit is now verbose by default, and shows changed filenames and the 
+      new revision number.  (Robert Collins, Martin Pool)
+
+    * Unify 'mv', 'move', 'rename'.  (#5379, Matthew Fuller)
+
   INTERNALS:
   
     * The internal storage of history, and logical branch identity have now

=== modified file 'bzrlib/builtins.py'
--- bzrlib/builtins.py	2006-03-16 20:53:32 +0000
+++ bzrlib/builtins.py	2006-03-22 12:05:37 +0000
@@ -326,42 +326,6 @@
                 print path
 
 
-class cmd_move(Command):
-    """Move files to a different directory.
-
-    examples:
-        bzr move *.txt doc
-
-    The destination must be a versioned directory in the same branch.
-    """
-    takes_args = ['source$', 'dest']
-    def run(self, source_list, dest):
-        tree, source_list = tree_files(source_list)
-        # TODO: glob expansion on windows?
-        tree.move(source_list, tree.relpath(dest))
-
-
-class cmd_rename(Command):
-    """Change the name of an entry.
-
-    examples:
-      bzr rename frob.c frobber.c
-      bzr rename src/frob.c lib/frob.c
-
-    It is an error if the destination name exists.
-
-    See also the 'move' command, which moves files into a different
-    directory without changing their name.
-    """
-    # TODO: Some way to rename multiple files without invoking 
-    # bzr for each one?"""
-    takes_args = ['from_name', 'to_name']
-    
-    def run(self, from_name, to_name):
-        tree, (from_name, to_name) = tree_files((from_name, to_name))
-        tree.rename_one(from_name, to_name)
-
-
 class cmd_mv(Command):
     """Move or rename a file.
 
@@ -376,6 +340,8 @@
     Files cannot be moved between branches.
     """
     takes_args = ['names*']
+    aliases = ['move', 'rename']
+
     def run(self, names_list):
         if len(names_list) < 2:
             raise BzrCommandError("missing file argument")
@@ -415,9 +381,14 @@
     takes_args = ['location?']
 
     def run(self, location=None, remember=False, overwrite=False, revision=None, verbose=False):
-        # FIXME: too much stuff is in the command class        
-        tree_to = WorkingTree.open_containing(u'.')[0]
-        stored_loc = tree_to.branch.get_parent()
+        # FIXME: too much stuff is in the command class
+        try:
+            tree_to = WorkingTree.open_containing(u'.')[0]
+            branch_to = tree_to.branch
+        except NoWorkingTree:
+            tree_to = None
+            branch_to = Branch.open_containing(u'.')[0] 
+        stored_loc = branch_to.get_parent()
         if location is None:
             if stored_loc is None:
                 raise BzrCommandError("No pull location known or specified.")
@@ -426,7 +397,6 @@
                 location = stored_loc
 
         br_from = Branch.open(location)
-        br_to = tree_to.branch
 
         if revision is None:
             rev_id = None
@@ -435,19 +405,22 @@
         else:
             raise BzrCommandError('bzr pull --revision takes one value.')
 
-        old_rh = br_to.revision_history()
-        count = tree_to.pull(br_from, overwrite, rev_id)
+        old_rh = branch_to.revision_history()
+        if tree_to is not None:
+            count = tree_to.pull(br_from, overwrite, rev_id)
+        else:
+            count = branch_to.pull(br_from, overwrite, rev_id)
 
-        if br_to.get_parent() is None or remember:
-            br_to.set_parent(location)
+        if branch_to.get_parent() is None or remember:
+            branch_to.set_parent(location)
         note('%d revision(s) pulled.' % (count,))
 
         if verbose:
-            new_rh = tree_to.branch.revision_history()
+            new_rh = branch_to.revision_history()
             if old_rh != new_rh:
                 # Something changed
                 from bzrlib.log import show_changed_revisions
-                show_changed_revisions(tree_to.branch, old_rh, new_rh)
+                show_changed_revisions(branch_to, old_rh, new_rh)
 
 
 class cmd_push(Command):
@@ -898,16 +871,27 @@
             # locations if the user supplies an extended path
             if not os.path.exists(location):
                 os.mkdir(location)
+        bzrdir.BzrDir.create_branch_convenience(location, format=format)
+
+
+class cmd_init_repository(Command):
+    """Create a shared repository to keep branches in."""
+    takes_args = ["location"] 
+    takes_options = [Option('format', 
+                            help='Use a specific format rather than the'
+                            ' current default format. Currently this '
+                            ' option only accepts "metadir" and "knit"',
+                            type=get_format_type)]
+    aliases = ["init-repo"]
+    def run(self, location, format=None):
+        from bzrlib.bzrdir import BzrDirMetaFormat1
+        from bzrlib.transport import get_transport
         if format is None:
-            # create default
-            bzrdir.BzrDir.create_standalone_workingtree(location)
-        else:
-            new_dir = format.initialize(location)
-            new_dir.create_repository()
-            new_dir.create_branch()
-            # TODO: ask the bzrdir format for the right classs
-            import bzrlib.workingtree
-            bzrlib.workingtree.WorkingTreeFormat3().initialize(new_dir)
+            format = BzrDirMetaFormat1()
+        get_transport(location).mkdir('')
+        newdir = format.initialize(location)
+        repo = newdir.create_repository(shared=True)
+        repo.set_make_working_trees(False)
 
 
 class cmd_diff(Command):
@@ -2413,6 +2397,8 @@
 # these get imported and then picked up by the scan for cmd_*
 # TODO: Some more consistent way to split command definitions across files;
 # we do need to load at least some information about them to know of 
-# aliases.
+# aliases.  ideally we would avoid loading the implementation until the
+# details were needed.
 from bzrlib.conflicts import cmd_resolve, cmd_conflicts, restore
 from bzrlib.sign_my_commits import cmd_sign_my_commits
+from bzrlib.weave_commands import cmd_weave_list, cmd_weave_join

=== modified file 'bzrlib/bzrdir.py'
--- bzrlib/bzrdir.py	2006-03-21 12:41:37 +0000
+++ bzrlib/bzrdir.py	2006-03-22 13:21:14 +0000
@@ -216,7 +216,8 @@
             return self.create_repository()
         
     @staticmethod
-    def create_branch_convenience(base, force_new_repo=False, force_new_tree=None):
+    def create_branch_convenience(base, force_new_repo=False,
+                                  force_new_tree=None, format=None):
         """Create a new BzrDir, Branch and Repository at the url 'base'.
 
         This is a convenience function - it will use an existing repository
@@ -238,13 +239,17 @@
         :param force_new_repo: If True a new repository is always created.
         :param force_new_tree: If True or False force creation of a tree or 
                                prevent such creation respectively.
+        :param format: Override for the for the bzrdir format to create
         """
         if force_new_tree:
             # check for non local urls
             t = get_transport(safe_unicode(base))
             if not isinstance(t, LocalTransport):
                 raise errors.NotLocalUrl(base)
-        bzrdir = BzrDir.create(base)
+        if format is None:
+            bzrdir = BzrDir.create(base)
+        else:
+            bzrdir = format.initialize(base)
         repo = bzrdir._find_or_create_repository(force_new_repo)
         result = bzrdir.create_branch()
         if force_new_tree or (repo.make_working_trees() and 
@@ -530,7 +535,8 @@
             source_branch.sprout(result, revision_id=revision_id)
         else:
             result.create_branch()
-        result.create_workingtree()
+        if result_repo is None or result_repo.make_working_trees():
+            result.create_workingtree()
         return result
 
 

=== modified file 'bzrlib/check.py'
--- bzrlib/check.py	2006-03-07 12:56:15 +0000
+++ bzrlib/check.py	2006-03-20 22:32:41 +0000
@@ -1,5 +1,4 @@
-# Copyright (C) 2004, 2005 by Martin Pool
-# Copyright (C) 2005 by Canonical Ltd
+# Copyright (C) 2005, 2006 by Canonical Ltd
 
 # This program is free software; you can redistribute it and/or modify
 # it under the terms of the GNU General Public License as published by
@@ -28,6 +27,11 @@
 # TODO: Get every revision in the revision-store even if they're not
 # referenced by history and make sure they're all valid.
 
+# TODO: Perhaps have a way to record errors other than by raising exceptions;
+# would perhaps be enough to accumulate exception objects in a list without
+# raising them.  If there's more than one exception it'd be good to see them
+# all.
+
 import bzrlib.ui
 from bzrlib.trace import note, warning
 from bzrlib.osutils import rename, sha_string, fingerprint_file
@@ -39,6 +43,8 @@
 class Check(object):
     """Check a branch"""
 
+    # The Check object interacts with InventoryEntry.check, etc.
+
     def __init__(self, branch):
         self.branch = branch
         self.repository = branch.repository
@@ -49,7 +55,7 @@
         self.missing_parent_links = {}
         self.missing_inventory_sha_cnt = 0
         self.missing_revision_cnt = 0
-        # maps (file-id, version) -> sha1
+        # maps (file-id, version) -> sha1; used by InventoryFile._check
         self.checked_texts = {}
         self.checked_weaves = {}
 

=== modified file 'bzrlib/commands.py'
--- bzrlib/commands.py	2006-03-16 12:53:16 +0000
+++ bzrlib/commands.py	2006-03-20 23:09:42 +0000
@@ -49,7 +49,13 @@
 
 
 def register_command(cmd, decorate=False):
-    "Utility function to help register a command"
+    """Utility function to help register a command
+
+    :param cmd: Command subclass to register
+    :param decorate: If true, allow overriding an existing command
+        of the same name; the old command is returned by this function.
+        Otherwise it is an error to try to override an existing command.
+    """
     global plugin_cmds
     k = cmd.__name__
     if k.startswith("cmd_"):
@@ -58,7 +64,7 @@
         k_unsquished = k
     if not plugin_cmds.has_key(k_unsquished):
         plugin_cmds[k_unsquished] = cmd
-        mutter('registered plugin command %s', k_unsquished)      
+        mutter('registered plugin command %s', k_unsquished)
         if decorate and k_unsquished in builtin_command_names():
             return _builtin_commands()[k_unsquished]
     elif decorate:
@@ -85,10 +91,9 @@
     builtins = bzrlib.builtins.__dict__
     for name in builtins:
         if name.startswith("cmd_"):
-            real_name = _unsquish_command_name(name)        
+            real_name = _unsquish_command_name(name)
             r[real_name] = builtins[name]
     return r
-
             
 
 def builtin_command_names():

=== modified file 'bzrlib/commit.py'
--- bzrlib/commit.py	2006-03-21 12:41:37 +0000
+++ bzrlib/commit.py	2006-03-22 13:21:14 +0000
@@ -128,6 +128,8 @@
 class ReportCommitToLog(NullCommitReporter):
 
     def snapshot_change(self, change, path):
+        if change == 'unchanged':
+            return
         note("%s %s", change, path)
 
     def completed(self, revno, rev_id):
@@ -229,6 +231,7 @@
         self.bound_branch = None
         self.local = local
         self.master_branch = None
+        self.master_locked = False
         self.rev_id = rev_id
         self.specific_files = specific_files
         self.allow_pointless = allow_pointless
@@ -391,6 +394,7 @@
         # so grab the lock
         self.bound_branch = self.branch
         self.master_branch.lock_write()
+        self.master_locked = True
 ####        
 ####        # Check to see if we have any pending merges. If we do
 ####        # those need to be pushed into the master branch
@@ -409,7 +413,8 @@
         """
         if not self.bound_branch:
             return
-        self.master_branch.unlock()
+        if self.master_locked:
+            self.master_branch.unlock()
 
     def _escape_commit_message(self):
         """Replace xml-incompatible control characters."""
@@ -499,7 +504,7 @@
         # mark-merge.  
         for path, ie in self.new_inv.iter_entries():
             previous_entries = ie.find_previous_heads(
-                self.parent_invs, 
+                self.parent_invs,
                 self.weave_store,
                 self.branch.repository.get_transaction())
             if ie.revision is None:

=== modified file 'bzrlib/hashcache.py'
--- bzrlib/hashcache.py	2006-02-10 12:31:18 +0000
+++ bzrlib/hashcache.py	2006-03-20 18:34:33 +0000
@@ -1,4 +1,4 @@
-# (C) 2005 Canonical Ltd
+# Copyright (C) 2005, 2006 by Canonical Ltd
 
 # This program is free software; you can redistribute it and/or modify
 # it under the terms of the GNU General Public License as published by
@@ -216,9 +216,12 @@
                 for fld in c[1]:
                     print >>outf, "%d" % fld,
                 print >>outf
-
             outf.commit()
             self.needs_write = False
+            mutter("write hash cache: %s hits=%d misses=%d stat=%d recent=%d updates=%d",
+                   self.cache_file_name(), self.hit_count, self.miss_count,
+                   self.stat_count,
+                   self.danger_count, self.update_count)
         finally:
             if not outf.closed:
                 outf.abort()

=== modified file 'bzrlib/inventory.py'
--- bzrlib/inventory.py	2006-03-11 08:50:10 +0000
+++ bzrlib/inventory.py	2006-03-21 12:26:54 +0000
@@ -328,6 +328,14 @@
 
         This is a template method, override _check for kind specific
         tests.
+
+        :param checker: Check object providing context for the checks; 
+             can be used to find out what parts of the repository have already
+             been checked.
+        :param rev_id: Revision id from which this InventoryEntry was loaded.
+             Not necessarily the last-changed revision for this file.
+        :param inv: Inventory from which the entry was loaded.
+        :param tree: RevisionTree for this entry.
         """
         if self.parent_id != None:
             if not inv.has_id(self.parent_id):
@@ -511,15 +519,14 @@
 class InventoryFile(InventoryEntry):
     """A file in an inventory."""
 
-    def _check(self, checker, rev_id, tree):
+    def _check(self, checker, tree_revision_id, tree):
         """See InventoryEntry._check"""
-        revision = self.revision
-        t = (self.file_id, revision)
+        t = (self.file_id, self.revision)
         if t in checker.checked_texts:
-            prev_sha = checker.checked_texts[t] 
+            prev_sha = checker.checked_texts[t]
             if prev_sha != self.text_sha1:
                 raise BzrCheckError('mismatched sha1 on {%s} in {%s}' %
-                                    (self.file_id, rev_id))
+                                    (self.file_id, tree_revision_id))
             else:
                 checker.repeated_text_cnt += 1
                 return
@@ -535,8 +542,8 @@
         else:
             w = tree.get_weave(self.file_id)
 
-        mutter('check version {%s} of {%s}', rev_id, self.file_id)
-        checker.checked_text_cnt += 1 
+        mutter('check version {%s} of {%s}', tree_revision_id, self.file_id)
+        checker.checked_text_cnt += 1
         # We can't check the length, because Weave doesn't store that
         # information, and the whole point of looking at the weave's
         # sha1sum is that we don't have to extract the text.

=== modified file 'bzrlib/merge.py'
--- bzrlib/merge.py	2006-03-18 23:40:51 +0000
+++ bzrlib/merge.py	2006-03-22 04:22:17 +0000
@@ -797,7 +797,7 @@
         and a conflict will be noted.
         """
         self._check_file(file_id)
-        lines = self._merged_lines(file_id)
+        lines = list(self._merged_lines(file_id))
         conflicts = '<<<<<<< TREE\n' in lines
         self.tt.create_file(lines, trans_id)
         if conflicts:

=== modified file 'bzrlib/reconcile.py'
--- bzrlib/reconcile.py	2006-03-17 13:03:04 +0000
+++ bzrlib/reconcile.py	2006-03-22 13:21:14 +0000
@@ -134,7 +134,7 @@
         self.repo.control_weaves.copy(self.inventory, 'inventory.backup', self.repo.get_transaction())
         self.pb.note('Backup Inventory created.')
         # asking for '' should never return a non-empty weave
-        new_inventory = self.repo.control_weaves.get_empty('inventory.new',
+        new_inventory_vf = self.repo.control_weaves.get_empty('inventory.new',
             self.repo.get_transaction())
 
         # we have topological order of revisions and non ghost parents ready.
@@ -142,25 +142,29 @@
         for rev_id in TopoSorter(self._rev_graph.items()).iter_topo_order():
             parents = self._rev_graph[rev_id]
             # double check this really is in topological order.
-            unavailable = [p for p in parents if p not in new_inventory]
+            unavailable = [p for p in parents if p not in new_inventory_vf]
             assert len(unavailable) == 0
             # this entry has all the non ghost parents in the inventory
             # file already.
             self._reweave_step('adding inventories')
-            # ugly but needed, weaves are just way tooooo slow else.
-            if isinstance(new_inventory, WeaveFile):
-                new_inventory._check_write_ok()
-                Weave._add_lines(new_inventory, rev_id, parents, self.inventory.get_lines(rev_id))
+            if isinstance(new_inventory_vf, WeaveFile):
+                # It's really a WeaveFile, but we call straight into the
+                # Weave's add method to disable the auto-write-out behaviour.
+                # This is done to avoid a revision_count * time-to-write additional overhead on 
+                # reconcile.
+                new_inventory_vf._check_write_ok()
+                Weave._add_lines(new_inventory_vf, rev_id, parents, self.inventory.get_lines(rev_id),
+                                 None)
             else:
-                new_inventory.add_lines(rev_id, parents, self.inventory.get_lines(rev_id))
+                new_inventory_vf.add_lines(rev_id, parents, self.inventory.get_lines(rev_id))
 
-        if isinstance(new_inventory, WeaveFile):
-            new_inventory._save()
-        # if this worked, the set of new_inventory.names should equal
+        if isinstance(new_inventory_vf, WeaveFile):
+            new_inventory_vf._save()
+        # if this worked, the set of new_inventory_vf.names should equal
         # self.pending
-        assert set(new_inventory.versions()) == self.pending
+        assert set(new_inventory_vf.versions()) == self.pending
         self.pb.update('Writing weave')
-        self.repo.control_weaves.copy(new_inventory, 'inventory', self.repo.get_transaction())
+        self.repo.control_weaves.copy(new_inventory_vf, 'inventory', self.repo.get_transaction())
         self.repo.control_weaves.delete('inventory.new', self.repo.get_transaction())
         self.inventory = None
         self.pb.note('Inventory regenerated.')
@@ -259,7 +263,7 @@
         self.repo.control_weaves.copy(self.inventory, 'inventory.backup', self.transaction)
         self.pb.note('Backup Inventory created.')
         # asking for '' should never return a non-empty weave
-        new_inventory = self.repo.control_weaves.get_empty('inventory.new',
+        new_inventory_vf = self.repo.control_weaves.get_empty('inventory.new',
             self.transaction)
 
         # we have topological order of revisions and non ghost parents ready.
@@ -267,19 +271,19 @@
         for rev_id in TopoSorter(self.revisions.get_graph().items()).iter_topo_order():
             parents = self.revisions.get_parents(rev_id)
             # double check this really is in topological order.
-            unavailable = [p for p in parents if p not in new_inventory]
+            unavailable = [p for p in parents if p not in new_inventory_vf]
             assert len(unavailable) == 0
             # this entry has all the non ghost parents in the inventory
             # file already.
             self._reweave_step('adding inventories')
             # ugly but needed, weaves are just way tooooo slow else.
-            new_inventory.add_lines(rev_id, parents, self.inventory.get_lines(rev_id))
+            new_inventory_vf.add_lines(rev_id, parents, self.inventory.get_lines(rev_id))
 
-        # if this worked, the set of new_inventory.names should equal
+        # if this worked, the set of new_inventory_vf.names should equal
         # self.pending
-        assert set(new_inventory.versions()) == set(self.revisions.versions())
+        assert set(new_inventory_vf.versions()) == set(self.revisions.versions())
         self.pb.update('Writing weave')
-        self.repo.control_weaves.copy(new_inventory, 'inventory', self.transaction)
+        self.repo.control_weaves.copy(new_inventory_vf, 'inventory', self.transaction)
         self.repo.control_weaves.delete('inventory.new', self.transaction)
         self.inventory = None
         self.pb.note('Inventory regenerated.')

=== modified file 'bzrlib/tests/blackbox/__init__.py'
--- bzrlib/tests/blackbox/__init__.py	2006-03-09 07:14:10 +0000
+++ bzrlib/tests/blackbox/__init__.py	2006-03-17 05:35:43 +0000
@@ -60,6 +60,7 @@
                      'bzrlib.tests.blackbox.test_revno',
                      'bzrlib.tests.blackbox.test_revision_info',
                      'bzrlib.tests.blackbox.test_selftest',
+                     'bzrlib.tests.blackbox.test_shared_repository',
                      'bzrlib.tests.blackbox.test_sign_my_commits',
                      'bzrlib.tests.blackbox.test_status',
                      'bzrlib.tests.blackbox.test_too_much',

=== modified file 'bzrlib/tests/blackbox/test_commit.py'
--- bzrlib/tests/blackbox/test_commit.py	2006-03-16 20:53:32 +0000
+++ bzrlib/tests/blackbox/test_commit.py	2006-03-20 20:02:21 +0000
@@ -1,5 +1,4 @@
 # Copyright (C) 2005, 2006 by Canonical Ltd
-# -*- coding: utf-8 -*-
 
 # This program is free software; you can redistribute it and/or modify
 # it under the terms of the GNU General Public License as published by
@@ -33,22 +32,54 @@
 
 class TestCommit(ExternalBase):
 
-    def test_empty_commit(self):
+    def test_05_empty_commit(self):
+        """Commit of tree with no versioned files should fail"""
+        # If forced, it should succeed, but this is not tested here.
         self.runbzr("init")
         self.build_tree(['hello.txt'])
         self.runbzr("commit -m empty", retcode=3)
-        self.runbzr("add hello.txt")
-        out,err = self.run_bzr("commit", "-m", "added")
-        self.assertEqual('', out)
-        self.assertEqual('added hello.txt\n'
-                         'Committed revision 1.\n',
+
+    def test_10_verbose_commit(self):
+        """Add one file and examine verbose commit output"""
+        self.runbzr("init")
+        self.build_tree(['hello.txt'])
+        self.runbzr("add hello.txt")
+        out,err = self.run_bzr("commit", "-m", "added")
+        self.assertEqual('', out)
+        self.assertEqual('added hello.txt\n'
+                         'Committed revision 1.\n',
+                         err)
+
+    def test_15_verbose_commit_with_unknown(self):
+        """Unknown files should not be listed by default in verbose output"""
+        # Is that really the best policy?
+        self.runbzr("init")
+        self.build_tree(['hello.txt', 'extra.txt'])
+        self.runbzr("add hello.txt")
+        out,err = self.run_bzr("commit", "-m", "added")
+        self.assertEqual('', out)
+        self.assertEqual('added hello.txt\n'
+                         'Committed revision 1.\n',
+                         err)
+
+    def test_16_verbose_commit_with_unchanged(self):
+        """Unchanged files should not be listed by default in verbose output"""
+        self.runbzr("init")
+        self.build_tree(['hello.txt', 'unchanged.txt'])
+        self.runbzr('add unchanged.txt')
+        self.runbzr('commit -m unchanged unchanged.txt')
+        self.runbzr("add hello.txt")
+        out,err = self.run_bzr("commit", "-m", "added")
+        self.assertEqual('', out)
+        self.assertEqual('added hello.txt\n'
+                         'Committed revision 2.\n',
                          err)
 
     def test_empty_commit_message(self):
         self.runbzr("init")
         file('foo.c', 'wt').write('int main() {}')
         self.runbzr(['add', 'foo.c'])
-        self.runbzr(["commit", "-m", ""] , retcode=3) 
+        self.runbzr(["commit", "-m", ""] , retcode=3)
 
     def test_other_branch_commit(self):
         # this branch is to ensure consistent behaviour, whether we're run

=== added file 'bzrlib/tests/blackbox/test_shared_repository.py'
--- bzrlib/tests/blackbox/test_shared_repository.py	1970-01-01 00:00:00 +0000
+++ bzrlib/tests/blackbox/test_shared_repository.py	2006-03-19 16:59:29 +0000
@@ -0,0 +1,55 @@
+# Copyright (C) 2005 by Canonical Ltd
+#
+# This program is free software; you can redistribute it and/or modify
+# it under the terms of the GNU General Public License as published by
+# the Free Software Foundation; either version 2 of the License, or
+# (at your option) any later version.
+#
+# This program is distributed in the hope that it will be useful,
+# but WITHOUT ANY WARRANTY; without even the implied warranty of
+# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+# GNU General Public License for more details.
+#
+# You should have received a copy of the GNU General Public License
+# along with this program; if not, write to the Free Software
+# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
+
+"""Black-box tests for repositories with shared branches"""
+
+import os
+
+from bzrlib.tests import TestCaseInTempDir
+import bzrlib.bzrdir
+import bzrlib.errors as errors
+
+class TestSharedRepo(TestCaseInTempDir):
+
+    def test_make_repository(self):
+        out, err = self.run_bzr("init-repository", "a")
+        self.assertEqual(out, "")
+        self.assertEqual(err, "")
+        dir = bzrlib.bzrdir.BzrDir.open('a')
+        self.assertIs(dir.open_repository().is_shared(), True)
+        self.assertRaises(errors.NotBranchError, dir.open_branch)
+        self.assertRaises(errors.NoWorkingTree, dir.open_workingtree)        
+
+    def test_init(self):
+        self.run_bzr("init-repo", "a")
+        self.run_bzr("init", "--format=metadir", "a/b")
+        dir = bzrlib.bzrdir.BzrDir.open('a')
+        self.assertIs(dir.open_repository().is_shared(), True)
+        self.assertRaises(errors.NotBranchError, dir.open_branch)
+        self.assertRaises(errors.NoWorkingTree, dir.open_workingtree)
+        bdir = bzrlib.bzrdir.BzrDir.open('a/b')
+        bdir.open_branch()
+        self.assertRaises(errors.NoRepositoryPresent, bdir.open_repository)
+        self.assertRaises(errors.NoWorkingTree, bdir.open_workingtree)
+
+    def test_branch(self):
+        self.run_bzr("init-repo", "a")
+        self.run_bzr("init", "--format=metadir", "a/b")
+        self.run_bzr('branch', 'a/b', 'a/c')
+        cdir = bzrlib.bzrdir.BzrDir.open('a/c')
+        cdir.open_branch()
+        self.assertRaises(errors.NoRepositoryPresent, cdir.open_repository)
+        self.assertRaises(errors.NoWorkingTree, cdir.open_workingtree)

=== modified file 'bzrlib/tests/test_commit.py'
--- bzrlib/tests/test_commit.py	2006-03-07 12:39:38 +0000
+++ bzrlib/tests/test_commit.py	2006-03-17 20:39:44 +0000
@@ -20,10 +20,12 @@
 import bzrlib
 from bzrlib.tests import TestCaseWithTransport
 from bzrlib.branch import Branch
+from bzrlib.bzrdir import BzrDir, BzrDirMetaFormat1
 from bzrlib.workingtree import WorkingTree
 from bzrlib.commit import Commit
 from bzrlib.config import BranchConfig
-from bzrlib.errors import PointlessCommit, BzrError, SigningFailed
+from bzrlib.errors import (PointlessCommit, BzrError, SigningFailed, 
+                           LockContention)
 
 
 # TODO: Test commit with some added, and added-but-missing files
@@ -376,4 +378,14 @@
                          wt.branch.repository.get_revision(
                             wt.branch.last_revision()).properties)
 
-
+    def test_safe_master_lock(self):
+        os.mkdir('master')
+        master = BzrDirMetaFormat1().initialize('master')
+        master.create_repository()
+        master_branch = master.create_branch()
+        master.create_workingtree()
+        bound = master.sprout('bound')
+        wt = bound.open_workingtree()
+        wt.branch.set_bound_location(os.path.realpath('master'))
+        master_branch.lock_write()
+        self.assertRaises(LockContention, wt.commit, 'silly')

=== modified file 'bzrlib/tests/test_hashcache.py'
--- bzrlib/tests/test_hashcache.py	2006-02-10 12:31:18 +0000
+++ bzrlib/tests/test_hashcache.py	2006-03-20 18:27:52 +0000
@@ -99,15 +99,12 @@
 
         self.assertEquals(hc.get_sha1('subdir'), None)
 
-        # it's likely neither are cached at the moment because they 
-        # changed recently, but we can't be sure
+        # pause briefly to make sure they're not treated as new uncacheable
+        # files
         pause()
 
-        # should now be safe to cache it if we reread them
         self.assertEquals(hc.get_sha1('foo'), sha1('g00dbye'))
-        self.assertEquals(len(hc._cache), 1)
         self.assertEquals(hc.get_sha1('foo2'), sha1('new content'))
-        self.assertEquals(len(hc._cache), 2)
 
         # write out, read back in and check that we don't need to
         # re-read any files

=== modified file 'bzrlib/tests/test_merge_core.py'
--- bzrlib/tests/test_merge_core.py	2006-02-27 20:48:40 +0000
+++ bzrlib/tests/test_merge_core.py	2006-03-22 04:22:17 +0000
@@ -260,9 +260,14 @@
         builder.add_file("3", "TREE_ROOT", "name5", "text3", True)
         builder.add_file("4", "TREE_ROOT", "name6", "text4", True)
         builder.remove_file("4", base=True)
-        builder.merge()
+        builder.add_file("5", "TREE_ROOT", "name7", "a\nb\nc\nd\ne\nf\n", True)
+        builder.change_contents("5", other="a\nz\nc\nd\ne\nf\n", 
+                                     this="a\nb\nc\nd\ne\nz\n")
+        builder.merge(merge_factory)
         self.assertEqual(builder.this.get_file("1").read(), "text4" )
         self.assertEqual(builder.this.get_file("2").read(), "text2" )
+        self.assertEqual(builder.this.get_file("5").read(), 
+                         "a\nz\nc\nd\ne\nz\n")
         self.assertIs(builder.this.is_executable("1"), True)
         self.assertIs(builder.this.is_executable("2"), False)
         self.assertIs(builder.this.is_executable("3"), True)

=== modified file 'bzrlib/transport/http/_pycurl.py'
--- bzrlib/transport/http/_pycurl.py	2006-03-16 20:02:36 +0000
+++ bzrlib/transport/http/_pycurl.py	2006-03-22 11:45:28 +0000
@@ -18,6 +18,11 @@
 
 # TODO: test reporting of http errors
 
+# TODO: Transport option to control caching of particular requests; broadly we
+# would want to offer "caching allowed" or "must revalidate", depending on
+# whether we expect a particular file will be modified after it's committed.
+# It's probably safer to just always revalidate.  mbp 20060321
+
 import os
 from StringIO import StringIO
 
@@ -119,11 +124,11 @@
         # There's no way in http/1.0 to say "must revalidate"; we don't want
         # to force it to always retrieve.  so just turn off the default Pragma
         # provided by Curl.
-        headers = ['Cache-control: must-revalidate',
-                   'Pragma:']
+        headers = ['Cache-control: max-age=0',
+                   'Pragma: no-cache']
         ## curl.setopt(pycurl.VERBOSE, 1)
-        # TODO: maybe show a summary of the pycurl version
-        ua_str = 'bzr/%s (pycurl)' % (bzrlib.__version__) 
+        # TODO: maybe include a summary of the pycurl version
+        ua_str = 'bzr/%s (pycurl)' % (bzrlib.__version__)
         curl.setopt(pycurl.USERAGENT, ua_str)
         curl.setopt(pycurl.HTTPHEADER, headers)
         curl.setopt(pycurl.FOLLOWLOCATION, 1) # follow redirect responses

=== modified file 'bzrlib/versionedfile.py'
--- bzrlib/versionedfile.py	2006-03-14 10:27:28 +0000
+++ bzrlib/versionedfile.py	2006-03-20 18:27:52 +0000
@@ -121,10 +121,10 @@
         Must raise RevisionNotPresent if any of the given parents are
         not present in file history.
         :param parent_texts: An optional dictionary containing the opaque 
-                             representations of some or all of the parents of 
-                             version_id to allow delta optimisations. 
-                             VERY IMPORTANT: the texts must be those returned
-                             by add_lines or data corruption can be caused.
+             representations of some or all of the parents of 
+             version_id to allow delta optimisations. 
+             VERY IMPORTANT: the texts must be those returned
+             by add_lines or data corruption can be caused.
         :return: An opaque representation of the inserted version which can be
                  provided back to future add_lines calls in the parent_texts
                  dictionary.

=== modified file 'bzrlib/weave.py'
--- bzrlib/weave.py	2006-03-14 10:27:28 +0000
+++ bzrlib/weave.py	2006-03-21 12:26:54 +0000
@@ -863,7 +863,8 @@
 
     def check(self, progress_bar=None):
         # TODO evaluate performance hit of using string sets in this routine.
-        # check no circular inclusions
+        # TODO: check no circular inclusions
+        # TODO: create a nested progress bar
         for version in range(self.num_versions()):
             inclusions = list(self._parents[version])
             if inclusions:
@@ -1366,13 +1367,12 @@
         w = readit()
         p = w.plan_merge(int(argv[3]), int(argv[4]))
         sys.stdout.writelines(w.weave_merge(p))
-            
     else:
         raise ValueError('unknown command %r' % cmd)
     
 
 
-def profile_main(argv): 
+def profile_main(argv):
     import tempfile, hotshot, hotshot.stats
 
     prof_f = tempfile.NamedTemporaryFile()

=== added file 'bzrlib/weave_commands.py'
--- bzrlib/weave_commands.py	1970-01-01 00:00:00 +0000
+++ bzrlib/weave_commands.py	2006-03-21 12:26:54 +0000
@@ -0,0 +1,57 @@
+# Copyright (C) 2006 by Canonical Ltd
+
+# This program is free software; you can redistribute it and/or modify
+# it under the terms of the GNU General Public License as published by
+# the Free Software Foundation; either version 2 of the License, or
+# (at your option) any later version.
+
+# This program is distributed in the hope that it will be useful,
+# but WITHOUT ANY WARRANTY; without even the implied warranty of
+# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+# GNU General Public License for more details.
+
+# You should have received a copy of the GNU General Public License
+# along with this program; if not, write to the Free Software
+# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
+
+"""builtin bzr commands relating to individual weave files
+
+These should never normally need to be used by end users, but might be
+of interest in debugging or data recovery.
+"""
+
+import sys
+
+from bzrlib.commands import Command
+
+class cmd_weave_list(Command):
+    """List the revision ids present in a weave, in alphabetical order"""
+
+    hidden = True
+    takes_args = ['weave_filename']
+
+    def run(self, weave_filename):
+        from bzrlib.weavefile import read_weave
+        weave = read_weave(file(weave_filename, 'rb'))
+        names = weave.versions()
+        names.sort()
+        print '\n'.join(names)
+
+
+class cmd_weave_join(Command):
+    """Join the contents of two weave files.
+
+    The resulting weave is sent to stdout.
+
+    This command is only intended for bzr developer use.
+    """
+
+    hidden = True
+    takes_args = ['weave1', 'weave2']
+
+    def run(self, weave1, weave2):
+        from bzrlib.weavefile import read_weave, write_weave
+        w1 = read_weave(file(weave1, 'rb'))
+        w2 = read_weave(file(weave2, 'rb'))
+        w1.join(w2)
+        write_weave(w1, sys.stdout)

